/*
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 * 
 * Contact Information:
 * Swarco Traffic Systems GmbH
 * Adolf-Dambach-Strae
 * 76571 Gaggenau
 * Phone: +49-7225-64-01
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */

package de.bsvrz.ste.stealgsba.algorithmen.lib;

import java.util.Date;
import java.util.Vector;

import de.bsvrz.dav.daf.main.ClientDavConnection;
import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.dav.daf.main.ClientReceiverInterface;
import de.bsvrz.dav.daf.main.ClientSenderInterface;
import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.DataDescription;
import de.bsvrz.dav.daf.main.DataNotSubscribedException;
import de.bsvrz.dav.daf.main.OneSubscriptionPerSendData;
import de.bsvrz.dav.daf.main.ReceiveOptions;
import de.bsvrz.dav.daf.main.ReceiverRole;
import de.bsvrz.dav.daf.main.ResultData;
import de.bsvrz.dav.daf.main.SendSubscriptionNotConfirmed;
import de.bsvrz.dav.daf.main.SenderRole;
import de.bsvrz.dav.daf.main.config.Aspect;
import de.bsvrz.dav.daf.main.config.AttributeGroup;
import de.bsvrz.dav.daf.main.config.ConfigurationObject;
import de.bsvrz.dav.daf.main.config.DataModel;
import de.bsvrz.dav.daf.main.config.DynamicObject;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.dav.daf.main.config.SystemObjectType;
import de.bsvrz.sys.funclib.debug.Debug;

/**
 * Diese abstrakte Klasse stellt Methoden und Variablen zur Verfgung fr die Kommunikation mit
 * dem DaV und dem Auswerten der empfangenen Attributgruppen durch einen separaten Thread.
 * @version $Revision: 1.3 $ / $Date: 2015/03/24 14:41:44 $ / ($Author: Asal $)
 * @author Swarco Traffic Systems GmbH
 * @author Matthias Obert
 */
public abstract class DaVKommunikation implements ClientReceiverInterface, Runnable, ClientSenderInterface
{
  /**
   * Comment for <code>debug</code>
   */
  private static final Debug m_debug = Debug.getLogger();

  /**
   * Verbindung zum DaV
   */
  protected ClientDavConnection m_connection;
  
  /**
   * Konfigurationsobjekt
   */
  protected ConfigurationObject m_konfigurationsObjekt;
  
  /**
   * Vector mit empfangenen Daten
   */
  private Vector<ResultData> m_resultInput = new Vector<ResultData>();
  /**
   * Flag Thread beenden.
   */
  private boolean m_threadBeenden = false;
  
  /**
   * Konstruktor der Klasse
   * @param connection Verbindung zum DaV
   * @param konfigurationsObjekt Konfigurationsobjekt 
   */
  public DaVKommunikation (ClientDavConnection connection, ConfigurationObject konfigurationsObjekt)
  {
    m_connection = connection;
    
    m_konfigurationsObjekt = konfigurationsObjekt;
  }

  
  /**
   * Konstruktor der Klasse
   * @param connection Verbindung zum DaV
   */
  public DaVKommunikation (ClientDavConnection connection)
  {
    m_connection = connection;
    
    m_konfigurationsObjekt = null;
  }

  /**
   * Methode die das Objekt mit der PID objPid beim Datenverteiler anmeldet zum
   * Lesen der Attibutgruppe atgPid unter dem Aspekt aspPid.
   *  
   * @param objPid Pid des Objekts. 
   * @param atgPid Attributgruppe die angemeldet werden soll.
   * @param aspPid Apekt der angemeldet werden soll.
   * @param role Rolle des Empfngers (siehe stauma.dav.clientside.ReceiveOptions).
   */
  public void anmeldeLeseDaV (String objPid, String atgPid, String aspPid, ReceiverRole role)
  {
//    String buffer = "Anmelden am DaV (Lesen): " + objPid + " " + atgPid + "  " + aspPid;
//    System.out.println(buffer);
    
    ClientDavInterface verb = m_connection;

    AttributeGroup atg = verb.getDataModel().getAttributeGroup(atgPid);

    Aspect asp = verb.getDataModel().getAspect(aspPid);

    DataDescription dd = new DataDescription(atg, asp);

    SystemObject obj = m_connection.getDataModel().getObject(objPid); 

    m_connection.subscribeReceiver(this, obj, dd, ReceiveOptions.normal(), role);
  }

  /**
   * Methode die das Objekt mit der PID objPid beim Datenverteiler der Attibutgruppe atgPid
   * unter dem Aspekt aspPid abmeldet.
   *  
   * @param objPid Pid des Objekts. 
   * @param atgPid Attributgruppe die angemeldet werden soll.
   * @param aspPid Apekt der angemeldet werden soll.
   */
  public void abmeldeLeseDaV (String objPid, String atgPid, String aspPid)
  {
//    String buffer = "Abmelden am DaV (Lesen): " + objPid + " " + atgPid + "  " + aspPid;
//    System.out.println(buffer);
    
    ClientDavInterface verb = m_connection;

    AttributeGroup atg = verb.getDataModel().getAttributeGroup(atgPid);

    Aspect asp = verb.getDataModel().getAspect(aspPid);

    DataDescription dd = new DataDescription(atg, asp);

    SystemObject obj = m_connection.getDataModel().getObject(objPid); 

    try
		{    	
    	if (obj.getType().isConfigurating())
    	{
    		m_konfigurationsObjekt = (ConfigurationObject)obj;
    	}
		}
		catch (Exception e)
		{
			e.printStackTrace();
		}
    
    m_connection.unsubscribeReceiver(this, obj, dd);
  }

  /**
   * Methode liest eine Attributgruppe direkt vom DaV und meldet sich anschliessen auf
   * das Update an
   * @param objPid Pid des Objekts 
   * @param atgPid Attributgruppe die angemeldet werden soll
   * @param aspPid Apekt der angemeldet werden soll
   * @param role Rolle des Empfngers (siehe stauma.dav.clientside.ReceiveOptions)
   */
  public void anmeldeLeseDirektUndUpdate (String objPid, String atgPid, String aspPid, ReceiverRole role)
  {
    leseDatenDirektVonDaV( objPid, atgPid, aspPid );
    
    anmeldeLeseDaV( objPid, atgPid, aspPid, role );
  }

  /**
   * Methode die das Objekt mit der PID objPid beim Datenverteiler anmeldet zum
   * Senden der Attibutgruppe atgPid unter dem Aspekt aspPid. 
   * @param objPid Pid des Objekts 
   * @param atgPid Pid der Attributgruppe
   * @param aspPid Pid des Aspekts
   */

  public DataDescription anmeldeSendeDaV (String objPid, String atgPid, String aspPid, SenderRole role)
  {
//    String buffer = "Anmelden am DaV (Senden): " + objPid + " " + atgPid + "  " + aspPid + " " + role.toString();
    
    ClientDavInterface verb = m_connection;
    
    AttributeGroup atg = verb.getDataModel().getAttributeGroup(atgPid);
    Aspect asp = verb.getDataModel().getAspect(aspPid);

    DataDescription dd = new DataDescription(atg, asp);

    SystemObject sysObj = verb.getDataModel().getObject(  objPid); 

    try
    {
      verb.subscribeSender( this, sysObj, dd, role);
    }
    catch ( OneSubscriptionPerSendData e1 )
    {
      e1.printStackTrace();
    }
    
    return dd;
  }

  /**
   * Methode versendet Daten vom Typ ResultData an den DaV
   * @param data zu versendende Daten
   * @param pidAspekt Pid des Aspekts
   * @param role Rolle
   */
  public void sendResultData (ResultData data, String pidAspekt, SenderRole role)
  {
    AttributeGroup atg = data.getDataDescription().getAttributeGroup();
    
    SystemObject sysObj = data.getObject();
    
    sendData ( data.getData(), sysObj.getPid(), atg.getPid(), pidAspekt, role );
  }

  /**
   * Methode versendet Daten vom Typ Data an den DaV
   * @param data zu versendende Daten
   * @param pidObjekt Pid des Objekts fr das die Daten versendet werden soll 
   * @param pidAttributgruppe Pid der Attributgruppe
   * @param pidAspekt Pid des Aspekts unter dem das Objekt versendet werden soll
   * @param role Rolle
   */
  public void sendData (Data data, String pidObjekt, String pidAttributgruppe, String pidAspekt, SenderRole role)
  {
    // Versenden der Attributgruppe

    if (data != null)
    {
      AttributeGroup atg = m_connection.getDataModel().getAttributeGroup (pidAttributgruppe);
      
      Aspect asp = m_connection.getDataModel().getAspect (pidAspekt);
      
      DataDescription dd = new DataDescription(atg, asp);

      SystemObject sysObj = m_connection.getDataModel().getObject(pidObjekt);

      ResultData dr = new ResultData( sysObj, dd, new Date ().getTime(), data );
      
//      Aspect asp = m_connection.getDataModel().getAspect (pidAspekt);
//      
//      AttributeGroup atg = data.getDataDescription().getAttributeGroup();
//      
//      DataDescription dd = new DataDescription(atg, asp);
//
//      SystemObject sysObj = data.getObject();
//
//      ResultData dr = new ResultData( data.getObject(), dd, new Date ().getTime(), data );

      // Es wird versucht die Daten zu senden, wenn dies fehlschlgt weil das Programm
      // sich noch nicht angemeldet hat, so erfolgt die Anmeldung und danach ein 
      // erneutes Senden.
      
      // 1. Sendeversuch
      
      try
      {
        m_connection.sendData( dr );
      }
      catch ( DataNotSubscribedException e1 )
      { 
        // (Erneutes) Anmelden am DaV
        
        anmeldeSendeDaV ( sysObj.getPid(), atg.getPid(), asp.getPid(), role );
        
//        System.out.println(this.getClass() + " --> " + sysObj.getPid() + " - " +  
//                           atg.getPid() + " - " + asp.getPid() + " - " + role );

        // ... und 2. Sendeversuch
        try
        {
          m_connection.sendData( dr );
        }
        catch ( DataNotSubscribedException e2 )
        {
          e2.printStackTrace();
          System.out.println(e2.toString());
        }
        catch ( SendSubscriptionNotConfirmed e2 )
        {
          e2.printStackTrace();
          System.out.println(e2.toString());
        }
      }
      catch ( SendSubscriptionNotConfirmed e1 )
      {
        e1.printStackTrace();
        System.out.println(e1.toString());
      }
    }
  }

  /**
   * Methode liest fr das Objekt mit der PID objPid die Attibutgruppe atgPid 
   * unter dem Aspekt aspPid direkt vom DaV 
   * @param objPid Pid des Objekts 
   * @param atgPid Attributgruppe die angemeldet werden soll
   * @param aspPid Apekt der angemeldet werden soll
   */
  public void leseDatenDirektVonDaV (String objPid, String atgPid, String aspPid)
  {
//    System.out.println("Lese direkt: " + objPid + " " + atgPid + " " + aspPid);
    
    ClientDavInterface verb = m_connection;

    AttributeGroup atg = verb.getDataModel().getAttributeGroup(atgPid);

    Aspect asp = verb.getDataModel().getAspect(aspPid);

    DataDescription dd = new DataDescription(atg, asp);

    SystemObject obj = verb.getDataModel().getObject(objPid); 

    try
    {
      ResultData data = verb.getData( obj, dd, 1000 );
      update ( new ResultData [] {data});
    }
    catch (Exception e) 
    {
      e.printStackTrace();
    }
  }

  /**
   * Update Methode des ClientReceiverInterface
   */
  public void update( ResultData[] results )
  {
    try 
    {
      for (int resultIndex = 0; resultIndex < results.length; ++resultIndex) 
      {
        ResultData result = results[resultIndex];

        synchronized (m_resultInput)
        {
          m_resultInput.add(result);
          m_resultInput.notifyAll();
        }
      }
    } 
    catch (Exception e) 
    {
      m_debug.error("" + e);
    }
  }
    
  /**
   * Run Methode des Threads zum Auswerten der vom DaV empfangenen Daten in einem eigenen Thread
   * (Entkoppeln von der update-Methode).
   */
  public void run()
  {
    while (true)
    {
    	if (isThreadBeenden())
    	{
    		System.out.println("Thread beenden -> " + this.toString());
    		break;
    	}
    	
      boolean flagDaten = false;
        
      synchronized (m_resultInput)
      {
        // Keine Daten in Queue vorhanden
        
        if (m_resultInput.size() < 1)
        {
          // in Wait Zustand gehen
          
          try
          {
            m_resultInput.wait();
  
            flagDaten = true;
          }
          catch (InterruptedException e)
          {
            m_debug.error("" + e);
          }
        }
        else
        {
          flagDaten = true;
        }
      }
        
      // Daten verarbeiten
      
      if (flagDaten)
      {
        verarbeiten();
      }
    }
  }

  /**
   * Methode zum eigentlichen Verarbeiten der empfangenen Daten des DaV. 
   */
  public void verarbeiten() 
  {
    ResultData[] results = null;
    
    int anzahl;

    while (true)
    {
      anzahl = 0;

      // Daten aus Eingangsqueue holen      
      
      synchronized (m_resultInput)
      {
        if (m_resultInput.size() > 0)
        {
          anzahl = m_resultInput.size();
          
          results = new ResultData[anzahl];
          
          for (int i = 0; i < anzahl; i++)
            results[i] = m_resultInput.remove(0);
        }
      }
      
      if (anzahl == 0)
        break;
      
      auswerteAttributgruppen (results);
    }
  }
  
  /**
   * Methode zum Auswerten der empfangenen Attributgruppen. Diese Methode muss
   * dann in den erweiternden Klassen spezifisch angepasst werden.
   * @param results empfangenen Attributgruppen
   */
  public abstract void auswerteAttributgruppen (ResultData[] results);

  /**
   * Methode bestimmt aus einem Systemobjekt ein Konfigurationsobjekt
   * @param connection Verbindung zum DaV
   * @param systemObjekt Systemobjekt
   * @return Konfigurationsobjekt
   */
  public static ConfigurationObject bestimmeKonfigurationsObjekt (ClientDavConnection connection, SystemObject systemObjekt)
  {
    if (connection == null)
      return null;
    
    String pid = systemObjekt.getPid();
    DataModel dataModel = connection.getDataModel();
    ConfigurationObject co = (ConfigurationObject)dataModel.getObject (pid);
    return co;
  }
  
  @Override
  public void dataRequest( SystemObject arg0, DataDescription arg1, byte arg2 )
  {
  }

  @Override
  public boolean isRequestSupported( SystemObject arg0, DataDescription arg1 )
  {
    return false;
  }

  /**
   * Methode liefert die Variable m_konfigurationsObjekt zurck
   * @return Variable m_konfigurationsObjekt
   */
  public ConfigurationObject getKonfigurationsObjekt()
  {
    return m_konfigurationsObjekt;
  }

  /**
   * Methode setzt die Variable m_konfigurationsObjekt
   * @param konfigurationsObjekt Wert auf den die Variable m_konfigurationsObjekt gesetzt werden soll
   */
  public void setKonfigurationsObjekt( ConfigurationObject konfigurationsObjekt )
  {
    m_konfigurationsObjekt = konfigurationsObjekt;
  }
  
  /**
   * Methode liefert das zu m_konfigurationsObjekt gehrende Systemobjekt zurck
   * @return das zu m_konfigurationsObjekt gehrende Systemobjekt
   */
  public SystemObject getSystemObjekt ()
  {
    return getKonfigurationsObjekt();
  }


	/**
	 * Gibt den Wert der Variable m_threadBeenden zurck.
	 *
	 * @return Wert der Variable m_threadBeenden.
	 */
	public boolean isThreadBeenden()
	{
		return m_threadBeenden;
	}


	/**
	 * Setzt den Wert der Variable m_threadBeenden.
	 *
	 * @param threadBeenden zu setzender Wert der Variable m_threadBeenden.
	 */
	public void threadBeenden()
	{
		m_threadBeenden = true;

		synchronized (m_resultInput)
		{
			m_resultInput.notifyAll();
		}
	}

}
