/*
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 * 
 * Contact Information:
 * Swarco Traffic Systems GmbH
 * Entwicklung
 * Adolf-Dambach-Strae
 * D-76571 Gaggenau
 * Phone: +49 (0)7225-64-0
 * Fax: +49 (0)7225-64-300
 * mailto: office.sts@swarco.de
 */

package de.bsvrz.ste.stealgsba.algorithmen.lib;

import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeSupport;

import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.ResultData;

/**
 * Diese Klasse dient zum Speichern der Daten die ber die Attributgruppe 'atg.kfzEinzelDatenFs'
 * ber den Datenverteiler empfangen wurden.<p>
 * 
 * ber Getter und Setter kann auf die Daten zugegriffen werden.
 *
 * @version $Revision: 1.2 $ / $Date: 2012/07/23 06:02:36 $ / ($Author: Asal $)
 *
 * @author Swarco Traffic Systems GmbH
 * @author Rolf Asal
 *
 */
public class KfzEinzelDatenFs
{
	/**
	 * Status der Messung.<br>
	 * 0 = ok
	 */
	private int m_status;	
	/**
	 * Fahrzeugklasse.
	 */
	private String m_fzKlasse;
	/**
	 * Geschwindigkeit vKfz.
	 */
	private int m_vKfz;	
	/**
	 * Fahrtrichtung.
	 */
	private String m_fahrtrichtung;
	/**
	 * Belegungszeit in Sekunden.
	 */
	private float m_belegzeit;
	/**
	 * Nettozeitlcke in Millisekunden.
	 */
	private int m_tNetto;
	/**
	 * Fahrzeuglnge in dm.
	 */
	private int m_fzLaenge;
	
  /**
   * Listener
   */
  private PropertyChangeSupport m_pcs = new PropertyChangeSupport (this);

  /**
   * Mutex fr den synchronisierten Zugriff auf die Parameter
   */
  private Object m_mutex = new Object();

	
	/**
	 * Konstruktor der Klasse.
	 */
	public KfzEinzelDatenFs()
	{		
	}
	
  /**
   * Methode fgt einen Listener der Klasse hinzu
   * @param pcl Listern der Klasse PropertyChangeListener
   */
  public void addPropertyChangeListener (PropertyChangeListener pcl)
  {
    m_pcs.addPropertyChangeListener( pcl );    
  }
  
  /**
   * Methode entfernt einen Listener aus der Klasse
   * @param pcl Listern der Klasse PropertyChangeListener
   */
  public void removePropertyChangeListener (PropertyChangeListener pcl)
  {
    m_pcs.removePropertyChangeListener( pcl );    
  }

  /**
   * Methode benachrichtigt die Listener
   */
  public void benachrichtigeListener ()
  {
    m_pcs.firePropertyChange( "Aktualisierung", null, null );
  }

  /**
   * Methode zum Auswerten der Attributgruppe 'atg.sswSbaUeMqZuordnungBereich'
   * @param dat empfangender Datensatz
   */
  void auswerteAttributgruppe(ResultData dat)
  {
    synchronized (m_mutex)
    {
      if (!dat.hasData())
      {
      	m_status = 1;
      	m_fzKlasse = null;
      	m_vKfz = -1;
      	m_fahrtrichtung = null;
      	m_belegzeit = -1;
      	m_tNetto = -1;
      	m_fzLaenge = -1;
      }
      else
      {
        Data data = dat.getData();
        
        m_status = data.getUnscaledValue("Status").intValue();
        m_fzKlasse = data.getScaledValue("FZKlasse").getValueText();
        m_vKfz = data.getUnscaledValue("vKfz").intValue();
        m_fahrtrichtung = data.getScaledValue("Fahrtrichtung").getValueText();
        m_belegzeit = data.getScaledValue("Belegzeit").floatValue();
        m_tNetto = data.getUnscaledValue("tNetto").intValue();
        m_fzLaenge = data.getUnscaledValue("FZLnge").intValue();        
      }
      
//      benachrichtigeListener();
    }
  }

  /**
   * Alle internen Variablen initialisieren.
   */
  public void init()
  {
  	m_status = 1;
  	m_fzKlasse = null;
  	m_vKfz = -1;
  	m_fahrtrichtung = null;
  	m_belegzeit = -1;
  	m_tNetto = -1;
  	m_fzLaenge = -1;
  }
  
  /**
	 * Gibt zurck, ob aktuelle Daten empfangen wurden.
	 * 
	 * @return true wenn Verkehrsdaten vorhanden, ansonsten false;
	 */
	public boolean hasDatenErhalten()
	{
		if ((m_status == -1) &&
				(m_fzKlasse == null) &&
				(m_vKfz == -1) &&
				(m_fahrtrichtung == null) &&
				(m_belegzeit == -1) &&
				(m_tNetto == -1) &&
				(m_fzLaenge == -1))
		{
			return false;
		}
		
		return true;
	}


	/* (non-Javadoc)
	 * @see java.lang.Object#toString()
	 */
	public String toString()
	{
		StringBuffer string = new StringBuffer();
		
		string.append(
				"\n Status        : " + m_status +
				"\n FZKlasse      : " + m_fzKlasse +
				"\n vKfz          : " + m_vKfz + " km/h " +
				"\n Fahrtrichtung : " + m_fahrtrichtung +
				"\n Belegzeit     : " + m_belegzeit + " s " +
				"\n tNetto        : " + m_tNetto + " ms " +
				"\n FZLnge       : " + m_fzLaenge + " m ");
		
		return string.toString();
	}

	/**
	 * Gibt den Status der Messung zurck.
	 * 
	 * @return Status der Messung.
	 */
	public int getStatus()
	{
		return m_status;
	}

	/**
	 * Gibt die Fahrzeugklasse zurck.
	 * 
	 * @return Fahrzeugklasse.
	 */
	public String getFzKlasse()
	{
		return m_fzKlasse;
	}

	/**
	 * Gibt die Geschwindigkeit vKfz zurck.
	 * 
	 * @return Geschwindigkeit vKfz.
	 */
	public int getvKfz()
	{
		return m_vKfz;
	}

	/**
	 * Gibt die Fahrtrichtung zurck.
	 * 
	 * @return Fahrtrichtung.
	 */
	public String getFahrtrichtung()
	{
		return m_fahrtrichtung;
	}

	/**
	 * Gibt die Belegzeit zurck.
	 * 
	 * @return Belegzeit.
	 */
	public float getBelegzeit()
	{
		return m_belegzeit;
	}

	/**
	 * Gibt die Nettozeitlcke in Millisekunden zurck.
	 * 
	 * @return Nettozeitlcke.
	 */
	public int gettNetto()
	{
		if (m_tNetto == 65535) // nicht ermittelbar
			return -1;

		return m_tNetto;
	}

	/**
	 * Gibt die Fahrzeuglnge zurck.
	 * 
	 * @return Fahrzeuglnge.
	 */
	public int getFzLaenge()
	{
		return m_fzLaenge;
	}
	
}
