/*
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 * 
 * Contact Information:
 * Swarco Traffic Systems GmbH
 * Entwicklung
 * Adolf-Dambach-Strae
 * D-76571 Gaggenau
 * Phone: +49 (0)7225-64-0
 * Fax: +49 (0)7225-64-300
 * mailto: office.sts@swarco.de
 */

package de.bsvrz.ste.stealgsba.algorithmen.lib;

import de.bsvrz.dav.daf.main.ClientDavConnection;
import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.dav.daf.main.ClientReceiverInterface;
import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.DataDescription;
import de.bsvrz.dav.daf.main.DataNotSubscribedException;
import de.bsvrz.dav.daf.main.ReceiveOptions;
import de.bsvrz.dav.daf.main.ReceiverRole;
import de.bsvrz.dav.daf.main.ResultData;
import de.bsvrz.dav.daf.main.config.Aspect;
import de.bsvrz.dav.daf.main.config.AttributeGroup;
import de.bsvrz.dav.daf.main.config.DataModel;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.sys.funclib.debug.Debug;

/**
 * Diese Klasse implementiert ein Objekt vom Typ 'typ.umfeldDatenSensor'.<p>
 * 
 * Die Klasse dient zur Verwaltung von Daten eines realen Umfelddatensensors.
 * 
 * ber Getter und Setter kann auf die Daten zugegriffen werden.
 *
 * @version $Revision: 1.2 $ / $Date: 2012/07/23 06:02:36 $ / ($Author: Asal $)
 *
 * @author Swarco Traffic Systems GmbH
 * @author Rolf Asal
 *
 */
public class UmfeldDatenSensorReal implements ClientReceiverInterface
{
	/**
   * Stellt Methoden fr die Debugausgabe zur Vefgung.
   */
	private Debug m_debug = Debug.getLogger();
	/**
	 * Aktuelle Verbindung zum Datenverteiler.
	 */
	private ClientDavConnection m_connection;
	/**
	 * Zugriff auf das aktuelle Datenmodell.
	 */
	private DataModel m_dataModel;
	/**
	 * Systemobjekt des Messquerschnitt.
	 */
	private SystemObject m_objekt;
	/**
	 * Attributgruppe fr den Datenempfang Sichtweitenstufe.
	 */
	private AttributeGroup m_atgUfdsFahrbahnOberflaechenZustand;
	/**
	 * Attributgruppe fr den Datenempfang Windrichtung.
	 */
	private AttributeGroup m_atgUfdsWindRichtung;
	/**
	 * Attributgruppe fr den Datenempfang Windgeschwindigkeit.
	 */
	private AttributeGroup m_atgUfdsWindGeschwindigkeit;
	/**
	 * Attributgruppe fr den Datenempfang Luftschadstoffe NOx.
	 */
	private AttributeGroup m_atgUfdsLuftschadstoffe;
	/**
	 * Attributgruppe fr den Datenempfang Lrmpegel.
	 */
	private AttributeGroup m_atgUfdsLaermpegel;
	/**
	 * Aspekt fr den Datenempfang der Sichtweitenstufe.
	 */
	private Aspect m_aspMesswertErsetzung;
	/**
	 * Unskalierter Wert UfdsFahrBahnOberFlchenZustand.
	 */
	private int m_fahrbahnOberfaechenZustand;
	/**
	 * Unskalierter Wert UfdsWindRichtung.
	 */
	private int m_windRichtung;
	/**
	 * Unskalierter Wert UfdsWindGeschwindigkeit.
	 */
	private float m_windGeschwindigkeit;	
	/**
	 * Skalierter Wert UfdsStickoxideNOx.
	 */
	private float m_stickoxideNOx;
	/**
	 * Skalierter Wert Lrmpegel.
	 */
	private float m_laermpegel;

	
	/**
	 * Konstruktor mit bernahme der Parameter.<p>
	 * 
	 * @param connection Aktuelle Verbindung zum Datenverteiler.
	 * @param objekt Systemobjekt Messquerschnitt.
	 */
	public UmfeldDatenSensorReal(ClientDavInterface connection, SystemObject objekt)
	{
		m_connection = (ClientDavConnection)connection;
		m_dataModel = m_connection.getDataModel();
		
		m_objekt = objekt;

		m_atgUfdsFahrbahnOberflaechenZustand = m_dataModel.getAttributeGroup("atg.ufdsFahrBahnOberFlchenZustand");
		m_atgUfdsWindGeschwindigkeit = m_dataModel.getAttributeGroup("atg.ufdsWindGeschwindigkeitMittelWert");
		m_atgUfdsWindRichtung = m_dataModel.getAttributeGroup("atg.ufdsWindRichtung");
		m_atgUfdsLuftschadstoffe = m_dataModel.getAttributeGroup("atg.ufdsStickoxideNOx");
		m_atgUfdsLaermpegel = m_dataModel.getAttributeGroup("atg.ufdsABewerteterSchallDruckPegel");

		m_aspMesswertErsetzung = m_dataModel.getAspect("asp.messWertErsetzung");
	}

	/**
	 * Anmelden der angemeldeten Daten am Datenverteiler.
	 */
	public void datenDavAnmelden()
	{
		if (m_objekt.getType().getPid().equals("typ.ufdsFahrBahnOberFlchenZustand"))
			empfaengerAnmeldenObjekt(this, m_objekt, m_atgUfdsFahrbahnOberflaechenZustand, m_aspMesswertErsetzung);
		else if (m_objekt.getType().getPid().equals("typ.ufdsWindGeschwindigkeitMittelWert"))
			empfaengerAnmeldenObjekt(this, m_objekt, m_atgUfdsWindGeschwindigkeit, m_aspMesswertErsetzung);
		else if (m_objekt.getType().getPid().equals("typ.ufdsWindRichtung"))
			empfaengerAnmeldenObjekt(this, m_objekt, m_atgUfdsWindRichtung, m_aspMesswertErsetzung);
		else if (m_objekt.getType().getPid().equals("typ.ufdsStickoxideNOx"))
			empfaengerAnmeldenObjekt(this, m_objekt, m_atgUfdsLuftschadstoffe, m_aspMesswertErsetzung);
		else if (m_objekt.getType().getPid().equals("typ.ufdsABewerteterSchallDruckPegel"))
			empfaengerAnmeldenObjekt(this, m_objekt, m_atgUfdsLaermpegel, m_aspMesswertErsetzung);		
	}
	
	/**
	 * Abmelden der angemeldeten Daten am Datenverteiler.
	 */
	public void datenDavAbmelden()
	{
		if (m_objekt.getType().getPid().equals("typ.ufdsFahrBahnOberFlchenZustand"))
			empfaengerAbmeldenObjekt(this, m_objekt, m_atgUfdsFahrbahnOberflaechenZustand, m_aspMesswertErsetzung);
		else if (m_objekt.getType().getPid().equals("typ.ufdsWindGeschwindigkeitMittelWert"))
			empfaengerAbmeldenObjekt(this, m_objekt, m_atgUfdsWindGeschwindigkeit, m_aspMesswertErsetzung);
		else if (m_objekt.getType().getPid().equals("typ.ufdsWindRichtung"))
			empfaengerAbmeldenObjekt(this, m_objekt, m_atgUfdsWindRichtung, m_aspMesswertErsetzung);
		else if (m_objekt.getType().getPid().equals("typ.ufdsStickoxideNOx"))
			empfaengerAbmeldenObjekt(this, m_objekt, m_atgUfdsLuftschadstoffe, m_aspMesswertErsetzung);
		else if (m_objekt.getType().getPid().equals("typ.ufdsABewerteterSchallDruckPegel"))
			empfaengerAbmeldenObjekt(this, m_objekt, m_atgUfdsLaermpegel, m_aspMesswertErsetzung);		
	}
	
	/**
   * Fr ein Objekt Attributgruppe/Aspekt Kombination am Datenverteiler anmelden.
   * 
   * @param instanz Instanz.
   * @param objekt Objekt.
   * @param atg Attributgruppe.
   * @param asp Aspekt.
   * @return true = Anmeldung erfolgreich, false = nicht erfolgreich.
   */
  private boolean empfaengerAnmeldenObjekt(ClientReceiverInterface instanz, SystemObject objekt, AttributeGroup atg, Aspect asp)
  {
    DataDescription dd;
    boolean ok = false;

    m_debug.fine(("DatenAnmeldenObjekt " + atg.getPid() + " " + asp.getPid() + " fr " + objekt));
    
    try
    {  
    	dd = new DataDescription(atg, asp);

    	m_connection.subscribeReceiver(instanz, objekt, dd, ReceiveOptions.normal(), ReceiverRole.receiver());
        
    	ok = true;
    } 
    catch (DataNotSubscribedException e)
    {
      m_debug.error(("Daten nicht zum Empfangen angemeldet " + e.getMessage()));
    } 
    
    return (ok);
  }
	/**
   * Fr ein Objekt Attributgruppe/Aspekt Kombination am Datenverteiler anmelden.
   * 
   * @param instanz Instanz.
   * @param objekt Objekt.
   * @param atg Attributgruppe.
   * @param asp Aspekt.
   * @return true = Anmeldung erfolgreich, false = nicht erfolgreich.
   */
  private boolean empfaengerAbmeldenObjekt(ClientReceiverInterface instanz, SystemObject objekt, AttributeGroup atg, Aspect asp)
  {
    DataDescription dd;
    boolean ok = false;

    m_debug.fine(("DatenAbmeldenObjekt " + atg.getPid() + " " + asp.getPid() + " fr " + objekt));
    
  	try
		{
			dd = new DataDescription(atg, asp);

			m_connection.unsubscribeReceiver(instanz, objekt, dd);
			  
			ok = true;
		}
		catch (Exception e)
		{
			m_debug.error(("Daten konnten nicht abgemeldet werden " + e.getMessage()));
		}
    
    return (ok);
  }

	/* (non-Javadoc)
	 * @see de.bsvrz.dav.daf.main.ClientReceiverInterface#update(de.bsvrz.dav.daf.main.ResultData[])
	 */
	@Override
	public void update(ResultData[] results)
	{
		for (int i = 0; i < results.length; i++)
		{
			ResultData resultData = results[i];
			
			// Fahrbahnoberflchenzustand
    	if (resultData.getDataDescription().getAttributeGroup().equals(m_atgUfdsFahrbahnOberflaechenZustand))
    	{ 
    		System.out.println("Fahrbahnoberflchenzustand -> " + resultData);

    		if (resultData.hasData())
    		{
    			Data data = resultData.getData();
    			m_fahrbahnOberfaechenZustand = data.getItem("FahrBahnOberFlchenZustand").getUnscaledValue("Wert").intValue();
    		}
    		else
    		{    			
    			m_fahrbahnOberfaechenZustand = -1; 
    		}
    	}
			// Windgeschwindigkeit
    	if (resultData.getDataDescription().getAttributeGroup().equals(m_atgUfdsWindGeschwindigkeit))
    	{ 
    		System.out.println("Windgeschwindigkeit -> " + resultData);

    		if (resultData.hasData())
    		{
    			Data data = resultData.getData();
    			m_windGeschwindigkeit = data.getItem("WindGeschwindigkeitMittelWert").getScaledValue("Wert").floatValue();
    		}
    		else
    		{    			
    			m_windGeschwindigkeit = -1; 
    		}
    	}
			// Windrichtung
    	if (resultData.getDataDescription().getAttributeGroup().equals(m_atgUfdsWindRichtung))
    	{ 
    		System.out.println("Windrichtung -> " + resultData);

    		if (resultData.hasData())
    		{
    			Data data = resultData.getData();
    			m_windRichtung = data.getItem("WindRichtung").getUnscaledValue("Wert").intValue();
    		}
    		else
    		{    			
    			m_windRichtung = -1; 
    		}
    	}

			// Luftschadstoffe NOx
    	if (resultData.getDataDescription().getAttributeGroup().equals(m_atgUfdsLuftschadstoffe))
    	{ 
    		System.out.println("Luftschadstoffe NOx -> " + resultData);

    		if (resultData.hasData())
    		{
    			Data data = resultData.getData();
    			m_stickoxideNOx = data.getItem("StickoxideNOx").getScaledValue("Wert").floatValue();
    		}
    		else
    		{    			
    			m_stickoxideNOx = -1; 
    		}
    	}

			// Laermpegel ABewerteterSchallDruckPegel
    	if (resultData.getDataDescription().getAttributeGroup().equals(m_atgUfdsLaermpegel))
    	{ 
    		System.out.println("Lrmpegel -> " + resultData);

    		if (resultData.hasData())
    		{
    			Data data = resultData.getData();
    			m_laermpegel = data.getItem("ABewerteterSchallDruckPegel").getScaledValue("Wert").floatValue();
    		}
    		else
    		{    			
    			m_laermpegel = -1; 
    		}
    	}

		}		
	}

	/**
	 * Gibt den unskalierten Wert fr den Fahrbahnoberffchenzustand zurck.
	 * 
	 * @return nskalierten Wert fr den Fahrbahnoberffchenzustand.
	 */
	public int getFahrbahnOberfaechenZustand()
	{
		return m_fahrbahnOberfaechenZustand;
	}
	/**
	 * Gibt das Objekt zurck.
	 * @return Objekt.
	 */
	public SystemObject getObjekt()
	{
		return m_objekt;
	}

	/**
	 * Gibt die Pid des Objekts zurck.
	 * @return Pid des Objekts.
	 */
	public String getPid()
	{
		if (m_objekt != null)
			return m_objekt.getPid();
		return null;
	}

	/**
	 * Gibt den unskalierten Wert fr die windrichtung zurck.
	 * 
	 * @return unskalierten Wert fr die Windrichtung.
	 */
	public int getWindRichtung()
	{
		return m_windRichtung;
	}

	/**
	 * Gibt den unskalierten Wert fr die Windgeschwindigkeit zurck.
	 * 
	 * @return unskalierten Wert fr die Windgeschwindigkeit.
	 */
	public float getWindGeschwindigkeit()
	{
		return m_windGeschwindigkeit;
	}

	/**
	 * Gibt den Skalierten Wert fr die Luftschadstoffe NOx zurck.
	 * 
	 * @return Skalierter Wert fr die Luftschadstoffe NOx
	 */
	public float getStickoxideNOx()
	{
		return m_stickoxideNOx;
	}

	/**
	 * Gibt den Skalierten Wert Lrmpegel zurck.
	 * @return Skalierter Wert Lrmpegel.
	 */
	public float getLaermpegel()
	{
		return m_laermpegel;
	}

}
