/*
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 * 
 * Contact Information:
 * Swarco Traffic Systems GmbH
 * Adolf-Dambach-Strae
 * 76571 Gaggenau
 * Phone: +49-7225-64-01
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */

package de.bsvrz.ste.stealgsba.algorithmen.verknuepfung;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;

import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.dav.daf.main.config.Aspect;
import de.bsvrz.dav.daf.main.config.DataModel;
import de.bsvrz.dav.daf.main.config.DynamicObject;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.dav.daf.main.config.SystemObjectType;
import de.bsvrz.ste.stealgsba.algorithmen.lib.Sequenz;
import de.bsvrz.ste.stealgsba.algorithmusrahmen.AlgorithmusrahmenVollausbau;
import de.bsvrz.ste.stealgsba.algorithmusrahmen.EinzelMassnahme;
import de.bsvrz.ste.stealgsba.algorithmusrahmen.Massnahmenanforderung;
import de.bsvrz.ste.stealgsba.algorithmusrahmen.Massnahmenbearbeitung;
import de.bsvrz.ste.stealgsba.algorithmusrahmen.Ursacheneinheit;
import de.bsvrz.ste.stealgsba.algorithmusrahmen.interfaces.IAlgorithmus;
import de.bsvrz.ste.stealgsba.algorithmusrahmen.interfaces.IMassnahmenbearbeitung;
import de.bsvrz.ste.stealgsba.algorithmusrahmen.interfaces.ISituationsbearbeitung;
import de.bsvrz.ste.stealgsba.algorithmusrahmen.interfaces.IUrsacheneinheitenListener;
import de.bsvrz.sys.funclib.application.StandardApplicationRunner;
import de.bsvrz.sys.funclib.debug.Debug;

/**
 * Diese Klasse implementiert der Steuerungsalgorithmus "Verknpfung"<p>
 * 
 * @version $Revision: 1.2 $ / $Date: 2015/03/25 06:13:52 $ / ($Author: Asal $)
 * @author Swarco Traffic Systems GmbH
 * @author Rolf Asal
 *
 */
public class VerknuepfungAlg extends AlgorithmusrahmenVollausbau implements IAlgorithmus, ISituationsbearbeitung, IMassnahmenbearbeitung, IUrsacheneinheitenListener
{
	/**
   * Stellt Methoden fr die Debugausgabe zur Vefgung.
   */
  private static Debug m_debug = Debug.getLogger();
  /**
   * Schnittstellen, die die logische Verbindung zum Datenverteiler reprsentiert. 
   */
	private ClientDavInterface m_connection;
	/**
	 * Schnittstelle zum Zugriff auf Objekte und Eigenschaften eines Datenmodells.
	 */
	private DataModel m_dataModel;
	/**
	 * Typ der zu bearbeitenden Ursacheneinheit.
	 */
	private final String m_typUrsacheneinheit = "typ.sswSbaUeVerknpfung";
	/**
	 * Liste der Ursacheneinheiten.
	 */
	private List<Ursacheneinheit> m_ursacheneinheiten = null;
	/**
	 * Aspekt zum Versand der Manahmenanforderung.
	 */
	private Aspect m_aspMassnahmeanforderung;	
	/**
	 * Objekt zur Synchornisation.
	 */
	private Object m_sync;	
	/**
	 * Objekt fr die Massnahmenbearbeitung.
	 */
	private Massnahmenbearbeitung m_mab;

	
	/**
	 * Konstruktor.<p>
	 * 
	 * Festlegung, fr welche Typen von Ursacheneineheiten der Algorithmus arbeiten soll.
	 * Instanziierung des Objekts fr die Massnahmenbearbeitung.
	 */
	public VerknuepfungAlg()
	{
		m_ursacheneinheitTypen.add(m_typUrsacheneinheit);
		
		m_mab = new Massnahmenbearbeitung(this, this);
	}
	
	/**
	 * Thread zur Bearbeitung der Manahmeanforderungen.
	 *
	 * @version $Revision: 1.2 $ / $Date: 2015/03/25 06:13:52 $ / ($Author: Asal $)
	 *
	 * @author Swarco Traffic Systems GmbH
	 * @author Mattihas Obert
	 *
	 */
	private class VerknuepfungThread implements Runnable
	{
		/**
		 * Default-Konstruktor ohne weitere Funktion.
		 */
		public VerknuepfungThread()
		{			
		}
		
		/* (non-Javadoc)
		 * @see java.lang.Runnable#run()
		 */
		public void run()
		{
			List<Massnahmenanforderung> mas = new ArrayList<Massnahmenanforderung>();

			while (true)
			{
				try
				{
					/**
					 * Warten bis Methode 'starteBerechnung' fertig.
					 */
					synchronized (m_sync)
					{
						m_sync.wait();
					}
				}
				catch (InterruptedException e1)
				{
					e1.printStackTrace();
				}
				
				System.out.println("VerknpfungThread.run() -> Massnahmenbearbeitung");

				/**
				 * Massnahmebearbeitung
				 */

				mas.clear();
				mas.addAll(m_mab.erzeugeMassnahmenanforderungen());

				for (Massnahmenanforderung massnahmenanforderung : mas)
				{
					if (massnahmenanforderung != null)
					{						
						massnahmenanforderung.setAbsenderAlgorithmus("Verknpfung");
						massnahmenanforderung.setAbsenderApplikation(getConnection().getLocalApplicationObject().getId());
						massnahmenanforderung.setNummer(Sequenz.next());
						
						massnahmenanforderung.setSpontan(isSpontan());
						massnahmenanforderung.setLoeschen(false);

						Ursacheneinheit ue = getUrsacheneinheit(massnahmenanforderung.getReferenz());
						if (ue != null)
						{
							if (ue.isBearbeitung())
							{
							  versendeMassnahmenanforderungAspekt(massnahmenanforderung, ue.getAspektVerwendung());
							}
							else
							{
								loescheMassnahmenanforderung(massnahmenanforderung);
							}
						}						
					}
				}
			}
		}
	}

	/**
	 * Statische Methode fr das Starten des Programms.
	 * 
	 * @param args Kommandozeilen-Parameter
	 */
	public static void main(String[] args)
	{
		StandardApplicationRunner.run(new VerknuepfungAlg(), args);
	}


	/* (non-Javadoc)
	 * @see de.bsvrz.ste.algorithmusrahmen.AlgorithmusrahmenGrundausbau#initialize(de.bsvrz.dav.daf.main.ClientDavInterface)
	 */
	@Override
	public void initialize(ClientDavInterface connection) throws Exception
	{
		super.initialize(connection);

		m_connection = connection;
		m_dataModel = m_connection.getDataModel();
		
		Sequenz.set(m_connection.getTime());

		m_sync = new Object();
		m_aspMassnahmeanforderung = getAspektVersand();
			
		this.setzeCallbackObjekt(this);
		
		System.out.println("VerknpfungAlg.initialize()");

		/**
		 * Thread starten
		 */	  
		Thread algorithmus = new Thread(new VerknuepfungThread());
//		algorithmus.setName(this.getClass().getName() + " " + new Date());
		algorithmus.setName("Massnahmenbearbeitung " + new Date());
		algorithmus.start();
	}

	/* (non-Javadoc)
	 * @see de.bsvrz.ste.algorithmusrahmen.interfaces.IAlgorithmus#starteBerechnung()
	 */
	public synchronized void starteBerechnung()
	{
		System.out.println("VerknpfungAlg.starteBerechnung()");
		
		if (m_ursacheneinheiten != null)
		{
			/**
			 * Situationsbearbeitung aller Ursacheneinheiten
			 */
			for (int i = 0; i < m_ursacheneinheiten.size(); i++)
			{
				VerknuepfungUe ueAlg = (VerknuepfungUe)m_ursacheneinheiten.get(i);
				
				if (ueAlg != null)
				{
					ueAlg.setZyklusLaenge(getZyklusLaenge());

					setzeStatus(ueAlg.berechnen(), (Ursacheneinheit)ueAlg);

					sendeUrsacheneinheitStatus(ueAlg);
				}
			}

			// Berechnung der Manahmenanforderung einleiten.
			synchronized (m_sync)
			{
				m_sync.notify();
			}		
		}
	}

	/**
	 * Gibt das Objekt Ursacheneinheit fr das angegebene SystemObjekt zurck.
	 * 
	 * @param objUe SystemObjekt der Ursacheneinheit.
	 * @return Objekt Ursacheneinheit oder <code>null</code>.
	 */
	private Ursacheneinheit getUrsacheneinheit(SystemObject objUe)
	{
		if (m_ursacheneinheiten != null)
		{
			for (int i = 0; i < m_ursacheneinheiten.size(); i++)
			{
				Ursacheneinheit ue = m_ursacheneinheiten.get(i);
				
				if (ue != null)
				{
					if (ue.getObjekt().equals(objUe))
						return ue;
				}
			}
		}
		return null;
	}

	/**
	 * Aktualisiert die interne Liste der Ursacheneinheiten.
	 */
	private void updateUrsachenEinheiten()
	{
		m_ursacheneinheiten = gibUrsacheneinheitenTyp(m_typUrsacheneinheit);
		
		System.out.println("Zu bearbeitenden UE's " + m_ursacheneinheiten);
	}


	/* (non-Javadoc)
	 * @see de.bsvrz.ste.algorithmusrahmen.interfaces.IUrsacheneinheitenListener#objektUrsacheneinheitErstellen(de.bsvrz.dav.daf.main.config.DynamicObject)
	 */
	public Ursacheneinheit objektUrsacheneinheitErstellen(DynamicObject obj)
	{
		if (obj != null)
		{
			SystemObjectType typ = obj.getType();
			
			if (typ.getPid().equals(m_typUrsacheneinheit))
			{
			  VerknuepfungUe ue = new VerknuepfungUe(getConnection(), (SystemObject)obj, getAspektVersand());
				ue.addListener(this);
				ue.datenAnmelden();
				
				System.out.println("UE " + ue);
				
				return (Ursacheneinheit)ue;
			}
		}
		return null;
	}

	/* (non-Javadoc)
	 * @see de.bsvrz.ste.algorithmusrahmen.AlgorithmusrahmenVollausbau#updateEinzelMassnahme(de.bsvrz.ste.algorithmusrahmen.Ursacheneinheit, de.bsvrz.ste.algorithmusrahmen.EinzelMassnahme)
	 */
	@Override
	public void updateEinzelMassnahme(Ursacheneinheit ue, EinzelMassnahme einzelMassnahme)
	{
		super.updateEinzelMassnahme(ue, einzelMassnahme);

		System.out.println("Verknpfung.updateEinzelMassnahme() " + ue);
	}

	/* (non-Javadoc)
	 * @see de.bsvrz.ste.algorithmusrahmen.interfaces.IUrsacheneinheitenListener#updateUrsachenEinheiten(de.bsvrz.ste.algorithmusrahmen.Ursacheneinheit[], de.bsvrz.ste.algorithmusrahmen.Ursacheneinheit[])
	 */
	public void updateUrsachenEinheiten(List<Ursacheneinheit> addedUrsacheneinheiten, List<Ursacheneinheit> removedUrsacheneinheiten)
	{
		System.out.println("Verknpfung.updateUrsachenEinheiten()");
		
		updateUrsachenEinheiten();
	}
}
