/*
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 * 
 * Contact Information:
 * Swarco Traffic Systems GmbH
 * Adolf-Dambach-Strae
 * 76571 Gaggenau
 * Phone: +49-7225-64-01
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */

package de.bsvrz.ste.stealgsba.algorithmen.verknuepfung;

import java.text.SimpleDateFormat;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import de.bsvrz.dav.daf.main.ClientDavConnection;
import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.dav.daf.main.ClientReceiverInterface;
import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.DataDescription;
import de.bsvrz.dav.daf.main.DataNotSubscribedException;
import de.bsvrz.dav.daf.main.ResultData;
import de.bsvrz.dav.daf.main.SendSubscriptionNotConfirmed;
import de.bsvrz.dav.daf.main.config.Aspect;
import de.bsvrz.dav.daf.main.config.AttributeGroup;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.ste.stealgsba.algorithmen.lib.EinzelsituationVorstufe;
import de.bsvrz.ste.stealgsba.algorithmen.lib.SituationAnalyseMq;
import de.bsvrz.ste.stealgsba.algorithmen.verknuepfung.ParameterVerknuepfung.Input;
import de.bsvrz.ste.stealgsba.algorithmen.verknuepfung.ParameterVerknuepfung.LogischeVerknuepfung;
import de.bsvrz.ste.stealgsba.algorithmusrahmen.UrsacheneinheitErweiterung;
import de.bsvrz.sys.funclib.debug.Debug;

/**
 * Klasse realisiert die Ursacheneinheit fr die Funktionalitt des Steuerungsalgorithmus 
 * "Verknpfung". 
 * 
 * @version $Revision: 1.1 $ / $Date: 2015/03/24 10:55:59 $ / ($Author: Asal $)
 * @author Swarco Traffic Systems GmbH
 * @author Rolf Asal
 *
 */
public class VerknuepfungUe extends UrsacheneinheitErweiterung implements ClientReceiverInterface
{
	/**
   * Stellt Methoden fr die Debugausgabe zur Vefgung.
   */
	private static Debug m_debug = Debug.getLogger();
	/**
	 * Systemobjekt der Ursacheneinheit.
	 */
	private SystemObject m_objekt;
	/**
	 * Attributgruppe fr den Empfang der globalen Parameter.
	 */
	private AttributeGroup m_atgParameter1;
	/**
	 * Aspekt fr den Empfang der globalen Parameter.
	 */
	private Aspect m_aspParameter;	
  /**
   * Parameter fr die Steuerungslogik Lrm.
   */
  private ParameterVerknuepfung m_paraVerknuepfung = new ParameterVerknuepfung();
  /**
   * Zugriff auf den Status der Einzelsituationvorstufen.
   */
  private Map<SystemObject, EinzelsituationVorstufe> m_einzelsituationVorstufe = Collections.synchronizedMap(new HashMap<SystemObject, EinzelsituationVorstufe>());  
	/**
	 * Attributgruppe fr den Versand der Analysedaten der Situationsbestimmung.
	 */
	private AttributeGroup m_atgAnalyse;
	/**
	 * Aspekt fr den Versand der Analysedaten.
	 */
	private Aspect m_aspAnalyse;
	
	/**
	 * Konstruktor mit bernahme der Parameter.<p>
	 * Anmelden zum Empfang der bentigten Parameter am Datenverteiler.
	 * 
	 * @param connection Aktuelle Verbindung zum Datenverteiler.
	 * @param ue Ursacheneinheit.
	 * @param aspektVersand Aspekt fr den Versand.
	 */
	public VerknuepfungUe(ClientDavInterface connection, SystemObject ue, Aspect aspektVersand)
	{
		super(connection, ue, aspektVersand);

		m_objekt = ue;
		
		m_atgParameter1 = m_dataModel.getAttributeGroup("atg.sswSbaUeVerknpfung");    
		m_aspParameter  = m_dataModel.getAspect("asp.parameterSoll");

		m_atgAnalyse = m_dataModel.getAttributeGroup("atg.sswSbaUeVerknpfungAnalyse");
		m_aspAnalyse = m_dataModel.getAspect("asp.analyse");
		
		// Daten zum Empfang anmelden		
		empfaengerAnmeldenObjekt(this, m_objekt, m_atgParameter1, m_aspParameter);

		// Daten zum Versand anmelden
		quelleAnmeldenObjekt(this, m_objekt, m_atgAnalyse, m_aspAnalyse);		
	}
	
	/**
	 * Methode berechnet die aktuelle Situation der Ursacheneinheit.
	 */
	public boolean berechnen()
	{
    SimpleDateFormat df = new SimpleDateFormat ("dd.MM.yyyy HH:mm:ss");

		boolean zurueck = false;
	  boolean einschaltKriterium = false;
	  int einschaltZaehler = 0;
	  
	  VerknuepfungAnalyse analyse = new VerknuepfungAnalyse();
	  
	  /**
	   * Status neu berechnen.
	   */
	  synchronized (m_einzelsituationVorstufe)
		{
	  	if ((m_einzelsituationVorstufe != null) &&
	  			(m_paraVerknuepfung != null))
	  	{
	  		System.out.println("VerknpfungUe.berechnen() -> " + df.format(m_connection.getTime()) + " UE='" + m_objekt.getName() + "' Logik=" + m_paraVerknuepfung.getLogischeVerknpfung());

	  		analyse.setLogik(m_paraVerknuepfung.getLogischeVerknpfung());
	  		
	  		Object[] keys = m_einzelsituationVorstufe.keySet().toArray();
	  		if (keys != null)
	  		{
	  			for (int i = 0; i < keys.length; i++)
					{
						SystemObject ue = (SystemObject)keys[i];
						
						EinzelsituationVorstufe esv = m_einzelsituationVorstufe.get(ue);
						
						Input input = m_paraVerknuepfung.getInput(ue);
						
						boolean statusInput = (input.isInputInvertiert()) ? !esv.isStatus() : esv.isStatus();

						System.out.printf("VerknpfungUe.berechnen() -> %-50s StatusUe=%-5s inputInvertriert=%-5s StatusInput=%-5s\n", ue.getName(), esv.isStatus(), input.isInputInvertiert(), statusInput);

						analyse.addInputAnalyse(ue, esv.isStatus(), input.isInputInvertiert(), statusInput);
						
						if (m_paraVerknuepfung.getLogischeVerknpfung().equals(LogischeVerknuepfung.UND))
						{
							if (statusInput)
							{
								einschaltZaehler++;
							}
						}
						else
						{
							einschaltKriterium |= statusInput;
						}
					}
	  		}
	  		
				if (m_paraVerknuepfung.getLogischeVerknpfung().equals(LogischeVerknuepfung.UND))
				{
		  		if (einschaltZaehler == m_einzelsituationVorstufe.size())
		  		{
		  			zurueck = true;
		  		}
				}
				else
				{
					if (einschaltKriterium)
					{
						zurueck = true;
					}
				}

				analyse.setStatus(zurueck);
	  	}
	    else
	    {
	    	System.out.println("Fehlerhafte Parameterversorgung: " + m_paraVerknuepfung);
	    	m_debug.fine("Fehlerhafte Parameterversorgung: " + m_paraVerknuepfung);
	    }
		}

	  sendeAnalyseDaten(analyse);
	  
    System.out.println("VerknpfungUe.berechnen() -> " + df.format(m_connection.getTime()) + " UE='" + m_objekt.getName() + "' aktueller Status=" + zurueck);
    
    return zurueck;
	}
	
	/**
	 * Diese Methode versendet die Analysedaten der Situationsbestimmung ber den Datenverteiler.
	 * 
	 * @param analyse Klasse mit den Analysedaten.
	 */
	private void sendeAnalyseDaten(VerknuepfungAnalyse analyse)
	{
		DataDescription dd = new DataDescription(m_atgAnalyse, m_aspAnalyse);

		Data data_neu = analyse.erzeugeDatensatz((ClientDavConnection)m_connection);

		ResultData erg = new ResultData(m_objekt, dd, m_connection.getTime(), data_neu);
		
		try
		{
			m_connection.sendData(erg);
		}
		catch (DataNotSubscribedException e)
		{
			e.printStackTrace();
		}
		catch (SendSubscriptionNotConfirmed e)
		{
			e.printStackTrace();
		}
	}
	


	/* (non-Javadoc)
	 * @see de.bsvrz.dav.daf.main.ClientReceiverInterface#update(de.bsvrz.dav.daf.main.ResultData[])
	 */
	public void update(ResultData results[])
	{
		System.out.println("VerknpfungUe.update()");
		
		for (int i = 0; i < results.length; i++)
		{
			ResultData resultData = results[i];
			
      if (resultData.getDataDescription().getAttributeGroup().equals(m_atgParameter1))
      { 
        System.out.println("-------> " + resultData);
        
        m_paraVerknuepfung.setdaten(resultData);

        System.out.println(m_paraVerknuepfung);
        m_debug.fine(m_paraVerknuepfung.toString());
        
        erzeugeObjekteEinzelsituationVorstufe();
      }
      
		}
	}

	/**
	 * Diese Methode erzeugt das Objekt fr die Verwaltung der Einzelsituationvorstufe entsprechend
	 * dem empfangenen Parameter.
	 */
	private void erzeugeObjekteEinzelsituationVorstufe()
	{
		synchronized (m_einzelsituationVorstufe)
		{			
			Object[] keys = m_einzelsituationVorstufe.keySet().toArray();
			if (keys != null)
			{
				for (int i = 0; i < keys.length; i++)
				{
					SystemObject ue = (SystemObject)keys[i];
					
					EinzelsituationVorstufe esv = m_einzelsituationVorstufe.get(ue);
					if (esv != null)
					{
						esv.abmeldeDav();
					}
				}
			}
			
			m_einzelsituationVorstufe.clear();
			
			if (m_paraVerknuepfung != null)
			{
				List<Input> input = m_paraVerknuepfung.getInput();
				if (input != null)
				{
					for (Input input2 : input)
					{
						SystemObject ue = input2.getUrsacheneinheit();
						if (ue != null)
						{
							if (!m_einzelsituationVorstufe.containsKey(ue))
							{
								m_einzelsituationVorstufe.put(ue, new EinzelsituationVorstufe((ClientDavConnection)m_connection, ue));
							}
						}
					}
				}
			}
		}
	}

}
